# constraint.nu -- 模块约束管理
#
# Copyright (C) 2022-2023 KAAAsS

use ../log.nu

use global_conf.nu

# 注册自定义约束
export def --env register_custom_constraint [
    name: string,        # 约束名
    fn: closure,         # 约束检查函数，返回 bool
] {
    log debug $"自定义约束: ($name): ($fn)"
    let constraints = (global_conf get_config "_" "constraints")
    global_conf set_config "_" "constraints" (
        $constraints | merge { $name: $fn }
    )
}

# 设置模块约束
export def --env require_constraints [
    module: string,
    constraints: list,
] {
    log debug $"声明约束: ($module): ($constraints)"

    # 检查约束存在
    $constraints | each { |$it| check_constraint --no-run $it }

    # 设置约束
    global_conf set_config $module "constraints" $constraints
}

# 检查某个约束
export def check_constraint [
    name: string,
    --no-run
] {
    let constraints = (global_conf get_config "_" "constraints")
    
    # 检查约束是否存在
    if (not ($name in $constraints)) {
        error make {
            msg: $"约束 ($name) 不存在",
            label: {
                text: "约束名",
                span: (metadata $name).span
            }
        }
    }

    # 运行
    if (not $no_run) {
        do ($constraints | get $name)
    }
}

# 检查所有存在的约束
export def check_all_constraints [] {
    let constraints = (global_conf get_config "_" "constraints")

    mut result = []
    for $cons in ($constraints | transpose key).key {
        $result = ($result | append {
            "constraint": $cons,
            "result": (check_constraint $cons),
        })
    }
    return $result
}
