# hook.nu -- 钩子管理
#
# Copyright (C) 2022-2023 KAAAsS

use ../log.nu
use ../pkg.nu
use ../interactive.nu [confirm]

use global_conf.nu
use module.nu [filtered_modules]

# 注册钩子
def --env register_hook [
    hook_type: string,   # 钩子类型
    module: string,      # 模块名称
    hook: closure,       # 运行块
] {
    log debug $"注册 ($hook_type): ($module) : ($hook)"

    let old_hooks = (global_conf get_config $module $hook_type null)

    # 合并新映射并设置
    global_conf set_config $module $hook_type (
        $old_hooks | append $hook
    )
}

# 注册模块安装前运行钩子
export def --env register_pre_install [module: string, hook: closure] {
    register_hook "pre_install" $module $hook
}

# 注册模块安装后运行钩子
export def --env register_post_install [module: string, hook: closure] {
    register_hook "post_install" $module $hook
}

# 注册模块卸载前运行钩子
export def --env register_pre_uninstall [module: string, hook: closure] {
    register_hook "pre_uninstall" $module $hook
}

# 注册模块卸载后运行钩子
export def --env register_post_uninstall [module: string, hook: closure] {
    register_hook "post_uninstall" $module $hook
}

# 声明包依赖
export def --env require_packages [
    module: string,
    pkgs: list,
] {
    log debug $"依赖: ($module): ($pkgs)"
    register_hook "install_dep" $module {||
        for $pkg_desc in $pkgs {
            let result = (pkg resolve_manager $pkg_desc)
            if ($result == null) {
                log debug $"包描述符 ($pkg_desc) 无可用的包管理器，跳过安装"
                continue
            }
            log debug $"解析包描述符：($pkg_desc) --> ($result)"
            pkg install --manager=($result.manager) $result.pkg
        }
    }
    register_hook "uninstall_dep" $module {||
        let remove = (confirm "- 是否卸载已安装的依赖包?" "n")
        if ($remove) {
            for $pkg in $pkgs {
                pkg uninstall $pkg
            }
        }
    }
}

# 运行指定钩子
export def do_hook [
    modules: list,
    hook_type: string,   # 钩子类型
    hook_name: string,   # 钩子名称
] {
    # 遍历模块
    for $module in (filtered_modules $modules) {
        let hooks = (global_conf get_config $module $hook_type)

        if ($hooks != null) {
            log status $"运行 ($module) 模块的($hook_name)..."

            for $hook in $hooks {
                do $hook
                null
            }
            log info ""
        }
    }
}
