# module.nu -- 模块管理
#
# Copyright (C) 2022-2023 KAAAsS

use ../log.nu

use global_conf.nu
use constraint.nu [check_constraint]

# 获得筛选后的模块列表
export def filtered_modules [
    want_modules?: list
] {
    let modules = (global_conf modules).name

    # 根据需要模块筛选
    let modules = if ($want_modules | is-empty) {
        $modules
    } else {
        # 检查模块是否存在
        mut result = []
        for $module in $want_modules {
            if (not ($module in $modules)) {
                log error $"模块 ($module) 不存在！"
                exit 1
            }
            $result = ($result | append $module)
        }
        $result
    }

    # 进行约束检查
    # TODO: 缓存检查结果
    if ("IGNORE_CONSTRAINT" in $env and $env.IGNORE_CONSTRAINT) {
        return $modules
    }
    mut result = []
    for $module in $modules {
        let constraints = (global_conf get_config $module "constraints" [])
        let check = (
            $constraints
                | each { |it|
                    let result = (check_constraint $it)
                    if (not $result) {
                        log debug $"模块 ($module) 不满足约束 ($it)"
                    }
                    $result
                }
                | reduce -f true { |it, acc| $it and $acc }
        )
        if ($check) {
            $result = ($result | append $module)
        }
    }
    return $result
}
