# apt.nu -- apt 管理器的实际实现
#
# Copyright (C) 2024 KAAAsS

# 检查包管理器是否可用
export def available? [] {
    not (which apt | is-empty) and not (which dpkg | is-empty)
}

# 检查包是否安装
export def check_install? [
    pkg_name: string
] {
    let ret = (
        do -i { dpkg -s $pkg_name } | complete   
    )
    $ret.exit_code == 0
}

# 安装包
export def install [
    pkg_name: string
] {
    let ret = (
        do -i { apt install $pkg_name -y } | complete
    )
    $ret.exit_code == 0
}

# 卸载包
export def uninstall [
    pkg_name: string
] {
    let ret = (
        do -i { apt remove $pkg_name -y } | complete
    )
    $ret.exit_code == 0
}

export def pack [] {
    {
        "name": "apt",
        "available?": { available? },
        "check_install?": {|p| check_install? $p },
        "install": {|p| install $p },
        "uninstall": {|p| uninstall $p }
    }
}
